<?php


namespace App\RestApi\Transactions\Controllers;

use Freelabois\LaravelQuickstart\Extendables\Resource;
use Freelabois\LaravelQuickstart\Traits\Crudable;
use Illuminate\Support\Facades\DB;
use Schaffen\Transaction\Transaction\Managers\TransactionManager;
use Schaffen\Transaction\Transaction\Models\Transaction;
use Schaffen\Transaction\Transaction\Repositories\TransactionRepository;
use Schaffen\Transaction\Transaction\Services\TransactionGenerate;
use Schaffen\Transaction\Transaction\Strategies\Commission\WithoutCommission;
use Schaffen\Transaction\Transaction\Strategies\Finances\WithoutFinance;
use Schaffen\Transaction\Transaction\Strategies\Invoices\WithoutInvoice;
use Schaffen\Transaction\Transaction\Strategies\PaymentForms\WithoutPayment;
use Schaffen\Transaction\Transaction\Strategies\WarehouseInventory\WithoutWarehouseInventory;


class TransactionsController
{

    use Crudable;

    protected $resource = Resource::class;

    public function __construct(TransactionManager $manager, TransactionRepository $repository)
    {
        $this->setup($manager, $repository);
    }


    /**
     * @return mixed
     * @throws \Exception
     */
    public function store()
    {
        $request = request();
        DB::beginTransaction();
        try {
            $sanitized = $request->input();
            $sanitized['trn_tipo'] = Transaction::PEDIDO_ORDEM_SERVICO;
            $pedido = $this->manager->execute(
                new TransactionGenerate(
                    new WithoutFinance(),
                    new WithoutPayment(),
                    new WithoutWarehouseInventory(),
                    new WithoutInvoice(),
                    new WithoutCommission()
                ),
                $sanitized);
            DB::commit();
        } catch (\Exception $e) {
            DB::rollBack();
            throw $e;
        }

        return new $this->resource($pedido);
    }
}
